#import "PSHChatViewController.h"
#import "PSHChatDAO.h"
#import "PSHEngine.h"
#import "PSHChatViewController.h"
#import "PSHABContactInDomainDAO.h"
#import "PSHGroupChatInfoDAO.h"
#import "PSHContactPickerViewController.h"

@interface PSHChatEngine : NSObject

+(instancetype)sharedInstance;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Initialization

/**
 *  Initializes the engine with Push Technologies app credentials and logging level. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId     Push Technologies application ID.
 *  @param appSecret Push Technologies application secret.
 *  @param logLevel  Level of logging detail (see `PSHLogLevel`).
 */
+(void)initializeWithAppId:(NSString*)appId appSecret:(NSString*)appSecret logLevel:(PSHLogLevel)logLevel;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param portNumber   Push host port number.
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                portNumber:(int)portNumber;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param portNumber   Push host port number. If port == 0, set default port (80 for HTTP, 443 for HTTPS).
 *  @param ssl          Use SSL.
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                portNumber:(int)portNumber
                       ssl:(BOOL)ssl;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param ssl          Use SSL.
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                       ssl:(BOOL)ssl;

/**
 *  Removes any data from the model.
 */
-(void)clean;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / User info

/**
 *  User's id.
 */
@property (nonatomic, readonly) NSString* userId;

/**
 *  User's API token.
 */
@property (nonatomic, readonly) NSString* userApiToken;

/**
 *  User's password (for web access).
 */
@property (nonatomic, readonly) NSString* userWebPassword;

/**
 *  User's current avatar image URL.
 */
@property (nonatomic, readonly) NSString* userAvatarURL;

/**
 *  User's current avatar thumbnail image URL.
 */
@property (nonatomic, readonly) NSString* userAvatarThumbnailURL;

/**
 *  Returns `YES` once **the app has been registered** and the app has enough privileges for the chat feature.
 */
@property (nonatomic, readonly) BOOL isChatEnabled;

/**
 *  Returns YES if current user in registered in chat domain.
 */
@property (nonatomic, readonly) BOOL isInDomain;

/**
 *  Returns universe URL.
 */
@property (nonatomic, readonly) NSString *universeURL;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Messages

/**
 *  Chat id that the user is currently on screen (`nil` if no chat view is currently visible).
 */
@property (nonatomic, strong) NSString* currentChatId;

/**
 *  Updates user's name.
 *
 *  @param name            New name.
 *  @param completionBlock Asynchronously executed block `PSHAsyncSimpleBlock`.
 */
-(void)updateUserName:(NSString*)name completion:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Updates user's avatar image. After a successful operation, both userAvatarUrl and userAvatarThumbnailUrl property values will be updated with the new URLs.
 *
 *  @param avatarImage     New avatar image.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)updateUserAvatar:(UIImage*)avatarImage completion:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Updates user's password (for web access). After a successful operation, userWebPassword will be updated with the new value.
 *
 *  @param webPassword     New password.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)updateUserWebPassword:(NSString*)webPassword completion:(PSHAsyncSimpleBlock)completionBlock;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Contacts

/**
 *  Returns an array of `PSHABContactInDomainDAO` instances with any user's Address book contact **registered in the chat domain**.
 */
@property (nonatomic, readonly) NSArray* contactList;

/**
 *  Returns an array of `PSHABContactInDomainDAO` instances with any user's Address book contact **registered in the chat domain**, but **still not a group member**.
 *
 *  @param groupChatId Group id
 *
 *  @return Array of PSHABContactInDomainDAO instances
 */
-(NSArray*)contactListNotInGroupChatWithId:(NSString*)groupChatId;

/**
 *  Returns an array of `PSHABContacDAO` instances with any user's Address book contact **NOT registered in the chat domain**.
 */
@property (nonatomic, readonly) NSArray* contactNotInDomainList;


/**
 *  Returns `YES` if chat list view is currently visible on screen. This value makes sense only if we're using `instantiateMainTabBarViewController`.
 */
@property (nonatomic, readonly) BOOL isChatListVisible;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Messages

/**
 *  Returns the number of chats with unread messages.
 */
@property (nonatomic, readonly) NSUInteger numberOfChatsWithUnreadMessages;

/**
 *  Returns the number of unread messages.
 */
@property (nonatomic, readonly) NSUInteger numberOfUnreadMessages;

/**
 *  Returns an array of `PSHChatDAO` instances with user's current chats.
 */
@property (nonatomic, readonly) NSArray* chatList;

/**
 *  Returns an instance of `PSHChatDAO` with the chat updated most recently.
 */
@property (nonatomic, readonly) PSHChatDAO * mostRecentChat;

/**
 *  Leaves a chat (either a single or group) given its id. The model is also updated if the call returns successfully.
 *
 *  @param id              Chat id.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)leaveChatWithId:(NSString*)ID completion:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Creates a new chat.
 *
 *  @param jid             JID of user or group to create chat with.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)createChatWith:(NSString*)jid completion:(PSHAsyncSimpleBlock)completionBlock;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Views

/**
 *  Returns an instance of the **whole app** (TabBar with contacts / chats / settings / campaigns).
 *
 *  @return `UITabBarController` instance.
 */
-(UITabBarController*)instantiateMainTabBarViewController;

/**
 *  Returns an instance of special view controller with registration form + secret validation.
 *
 *  @return `UINavigationController` instance, the root view controller is the registration form.
 */
-(UINavigationController*)instantiateRegistrationNavigationController;

/**
 *  Returns an instance `PSHChatViewController` which presents a chat conversation.
 *
 *  @return `PSHChatViewController` instance.
 */
-(PSHChatViewController*)instantiateChatViewController;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Registration

/**
 *  Asks Push Technologies SDK to send the user (eg. by SMS) the secret word. This is the first step of the chat registration process.
 *
 *  @param phoneNumber     User's phone number.
 *  @param callingCode     User's country calling code (prefix, eg. 34 for Spain).
 *  @param completionBlock Asynchronously executed block.
 */
-(void)requestUserSecretForPhoneNumber:(NSString*)phoneNumber
                           callingCode:(NSString*)callingCode
                            completion:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Tries to register the user with a provided info and the secret he (should have) received in the previous registration step.
 *
 *  @param name            User's name.
 *  @param phoneNumber     Phone number.
 *  @param callingCode     Country calling code (phone pregix).
 *  @param countryCode     Country code.
 *  @param countryName     Country name.
 *  @param secret          Secret word from 1st step.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)registerUserWithName:(NSString*)name
                phoneNumber:(NSString*)phoneNumber
                callingCode:(NSString*)callingCode
                countryCode:(NSString*)countryCode
                countryName:(NSString*)countryName
                     secret:(NSString*)secret
                 completion:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Generates a PSHModel with the provided information on a dictionary.
 *
 *  @param user_data       User data dictionary.
 */
-(void)registerUserWithDictionary:(NSDictionary *)user_data;

/**
 *  Generates a PSHModel with the provided information by components.
 *
 *  @param api_token       Api token.
 *  @param jid             JID.
 *  @param name            Name.
 */
-(void)registerUserWithToken:(NSString *)api_token
                         jid:(NSString *)jid
                        name:(NSString *)name;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Invitations

/**
 *  Sends SMS invitations to AddressBook contacts.
 *
 *  @param contacts        An array of `PSHABContactDAO` instances.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)sendInvitationToContacts:(NSArray*)contacts completion:(PSHAsyncSimpleBlock)completionBlock;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Groups

/**
 *  Creates a group chat.
 *
 *  @param name            Group name.
 *  @param members         Array of `PSHABContactInDomainDAO` instances, one for each group member.
 *  @param avatarData      Group avatar image data.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)requestCreateGroupWithName:(NSString*)name members:(NSArray*)members avatar:(NSData*)avatarData completionBlock:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Adds a contact to a group chat.
 *
 *  @param groupId         Group id
 *  @param contact         The contact in domain who is going to be added.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)requestInviteToGroupWithId:(NSString*)groupId contact:(PSHABContactInDomainDAO *)contact completionBlock:(PSHAsyncSimpleBlock)completionBlock;

/**
 *  Block for asynchronous operations with no one argument (argument class depends on the call).
 *
 *  @param PSHGroupChatInfoDAO* Instance of `PSHGroupChatInfoDAO` with the requested group information, or `nil` if an error occurred.
 *  @param NSError* An error if the operation fails; otherwise (success), `nil`.
 *
 */
typedef void(^PSHGroupInfoAsyncBlock)(PSHGroupChatInfoDAO *, NSError*);

/**
 *  Gets chat group info.
 *
 *  @param groupId         Group id
 *  @param completionBlock Asynchronously executed block, fired once the operation has finished
 */
-(void)requestGroupInfoWithId:(NSString*)groupId completion:(PSHGroupInfoAsyncBlock)completionBlock;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Chat / Contacts

/**
 *  Adds new contacts to the list.
 *
 *  @param contactList     Array of strings, each one being a phone number.
 *  @param completionBlock Asynchronously executed block.
 */
-(void)addContactsFromArray:(NSArray *)contactList completion:(PSHAsyncSimpleBlock)completionBlock;

@end
