#import <MediaPlayer/MediaPlayer.h>
#import <AddressBookUI/AddressBookUI.h>

@class PSHChatViewController;

/**
 *  `PSHChatViewDelegate` provides callbacks for events triggered by an instance of `PSHChatViewController`, or in other words: app user is sending/reading messages to/from other users in a chat. Developers must set the `PSHChatViewController` instance `delegate` property and implement its methods in order to handle those events and offer to the user the desired chat behaviour. They also must take into account that whenever `instantiateMainTabBarViewController` method of `PSHEngine` is used to present all the tabbed view, the SDK **assumes that app delegate implements `PSHChatViewDelegate` methods**.
 */
@protocol PSHChatViewDelegate <NSObject>

/**
 *  Triggered from the `viewDidLoad` as any `UIViewController`. When this method is callled is time to set `PSHChatViewController` instance `shareActivities` property (see `PSHChatViewControllerShareActivity`).
 *
 *  @param chatView `PSHChatViewController` instance.
 */
-(void)PSHChatViewDidLoad:(PSHChatViewController*)chatView;

/**
 *  Triggered whenever the user taps on an image message.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param imageURL Image URL.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView didTapOnImageWithURL:(NSURL*)imageURL;

/**
 *  Triggered whenever the user taps on a link inside a text message.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param url      Link URL.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView didSelectLinkWithURL:(NSURL*)url;

/**
 *  Triggered whenever the user taps on a phone number inside a text message.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param url      Phone number boxed as an `NSURL` instance.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView didSelectLinkWithPhoneNumber:(NSURL*)url;

/**
 *  Triggered whenever the user taps on a shared contact message. Developers should push the argument `ABPersonViewController` instance ideally in a `UINavigationController`.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newPersonViewController An instance of `ABPersonViewController` with the shared contact data.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView shouldPushContactViewController:(ABPersonViewController*)newPersonViewController;

/**
 *  Triggered whenever the user taps on a shared location message. Developers should push the argument `UIViewController` instance ideally in a `UINavigationController`.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newViewController An instance of `UIViewController` with a map with the shared location.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView shouldPushLocationViewController:(UIViewController*)newViewController;

/**
 *  Triggered whenever the user taps on save contact.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newPersonViewController An instance of `ABPersonViewController` with the shared contact data.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView savedContactViewController:(ABPersonViewController*)newPersonViewController;

/**
 *  Triggered whenever the user tries to share a contact with other user/s. Developers should present the argument `ABPeoplePickerNavigationController` instance.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newPeoplerPickerNavigationController An instance of `ABPeoplePickerNavigationController` to present
 */
-(void)PSHChatView:(PSHChatViewController*)chatView shouldPresentPeoplePickerNavigationController:(ABPeoplePickerNavigationController*)newPeoplerPickerNavigationController;

/**
 *  Triggered whenever the user tries to share an image or video with other user/s. Developers should present the argument `UIImagePickerController` instance.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newImagePickerController An instance of `ABPeoplePickerNavigationController` to present.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView shouldPresentUIImagePickerController:(UIImagePickerController*)newImagePickerController;

/**
 *  Triggered whenever the user taps on a video message play icon. Developers should present the argument `MPMoviePlayerViewController` instance.
 *
 *  @param chatView `PSHChatViewController` instance.
 *  @param newPlayerViewController An instance of `MPMoviePlayerViewController` to present.
 */
-(void)PSHChatView:(PSHChatViewController*)chatView presentMoviePlayerViewControllerAnimated:(MPMoviePlayerViewController*)newPlayerViewController;

@end

/**
 *  A share activity is just a relationship between a choice and an action to be performed after the user picked that choice. The choices are presented using the standard `UIActionSheet` when app user taps the plus sign on the right side of the text input area in the chat view.
 */
@interface PSHChatViewControllerShareActivity : NSObject

/**
 *  Creates a new instance of `PSHChatViewControllerShareActivity`.
 *
 *  @param text     Text for the `UIActionSheet` button.
 *  @param callback Action to be performed after users taps the choice button.
 *
 *  @return `PSHChatViewControllerShareActivity` instance given a choice / action.
 */
-(id)initWithText:(NSString*)text andCallback:(void(^)(PSHChatViewController*))callback;

@end

/**
 *  `PSHChatViewController` presents a conversation with *bubbled* messages with useful features such a text input area which shrinks or expands itself depending on the text size or keyboard and orientation changes awareness.
 */
@interface PSHChatViewController : UIViewController

/**
 *  `PSHChatViewDelegate` for the `PSHChatViewController`. Developers must set this property before the view is presented whenever the view is instantiated from `PSHEngine` method `instantiateMainTabBarViewController`. Otherwise (`instantiateChatViewController`), the application delegate is assumed as the instance delegate.
 */
@property (nonatomic, weak) id<PSHChatViewDelegate> delegate;

/**
 *  Chat id for the conversation.
 */
@property (nonatomic, strong) NSString* chatId;

/**
 *  An array of `PSHChatViewControllerShareActivity` instances.
 */
@property (nonatomic, strong) NSArray* shareActivities;

/**
 *  Title of chat view.
 */
@property (nonatomic, strong) NSString* chatTitle;

/**
 *  Starts the flow to let the user share his/her location.
 */
-(void)performSendLocation;

/**
 *  Starts the flow to let the user share a contact from his/her Address Book.
 */
-(void)performSendContact;

/**
 *  Starts the flow to let the user take and share a picture.
 */
-(void)performTakeAndSendPicture;

/**
 *  Starts the flow to let the user take and share a video.
 */
-(void)performTakeAndSendVideo;

/**
 *  Starts the flow to let the user pick a picture or video from his/her Camera Roll.
 */
-(void)performPickMediaFromCameraRollAndSend;

/**
 *  Lock chat textfield keyboard.
 */
- (void)lockKeyboard;

/**
 *  Unlock chat textfield keyboard.
 */
- (void)unlockKeyboard;

@end