#import "PSHCampaignDAO.h"
#import "PSHCustomDAO.h"

/**
 *  Asynchronously executed block triggered after an API call.
 *
 *  @param `NSError` An error if the operation fails; otherwise (success), `nil`
 */
typedef void(^PSHAsyncSimpleBlock)(NSError*);

/**
 *  Defined type to configure the SDK logging level.
 */
typedef NS_ENUM(NSInteger, PSHLogLevel) {
    /**
     *  Logging is OFF
     */
    PSHLogLevelNone,
    /**
     *  Standard log level ALERT
     */
    PSHLogLevelAlert,
    /**
     *  Standard log level CRITICAL
     */
    PSHLogLevelCritical,
    /**
     *  Standard log level ERROR
     */
    PSHLogLevelError,
    /**
     *  Standard log level WARNING
     */
    PSHLogLevelWarning,
    /**
     *  Standard log level NOTICE
     */
    PSHLogLevelNotice,
    /**
     *  Standard log level INFO
     */
    PSHLogLevelInfo,
    /**
     *  Standard log level DEBUG (DEFAULT)
     */
    PSHLogLevelDebug
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 *  `PSHEngine` brings developers a tool to interact with Push Technologies API and query the model.<br />
 *  All the API calls are performed in the background. Methods involved in API queries accept block parameter of type `PSHAsyncSimpleBlock` which signature is:
 *
 *  <pre>typedef void(^PSHAsyncSimpleBlock)(NSError*)</pre>
 */

@interface PSHEngine : NSObject


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Initialization

/**
 *  Initializes the engine with Push Technologies app credentials and logging level. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId     Push Technologies application ID.
 *  @param appSecret Push Technologies application secret.
 *  @param logLevel  Level of logging detail (see `PSHLogLevel`).
 */
+(void)initializeWithAppId:(NSString*)appId appSecret:(NSString*)appSecret logLevel:(PSHLogLevel)logLevel;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param portNumber   Push host port number. If port == 0, set default port (80 for HTTP).
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                portNumber:(int)portNumber;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param portNumber   Push host port number. If port == 0, set default port (80 for HTTP, 443 for HTTPS).
 *  @param ssl          Use SSL.
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                portNumber:(int)portNumber
                       ssl:(BOOL)ssl;

/**
 *  Initializes the engine with Push Technologies app credentials, logging level and push host. Should be called ideally inside `application:didFinishLaunchingWithOptions:`
 *  and before any other method or property of the SDK.<br/>
 *  The first time this method is being called will trigger a `PSHSuccessfulAppRegistrationBusEvent` bus event (see `PSHBusProvider`).
 *
 *  @param appId        Push Technologies application ID.
 *  @param appSecret    Push Technologies application secret.
 *  @param logLevel     Level of logging detail (see `PSHLogLevel`).
 *  @param hostName     Push host name.
 *  @param ssl          Use SSL.
 */
+(void)initializeWithAppId:(NSString*)appId
                 appSecret:(NSString*)appSecret
                  logLevel:(PSHLogLevel)logLevel
                  hostName:(NSString *)hostName
                       ssl:(BOOL)ssl;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Singleton

/**
 *  Gets the singleton engine instance. Should be called once the engine has been initialized (see `initializeWithAppId:appSecret:logLevel`).
 *
 *  @return Shared `PSHEngine` instance. Returns nil if the engine was not previously initialized.
 */
+(PSHEngine*)sharedInstance;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name SDK State

/**
 *  Returns `YES` when the app has been previously registered, that is when local model has been updated with the necessary info from Push Technologies platform in order to perform any operation.
 */
@property (nonatomic, readonly) BOOL isAppRegistered;

/**
 *  Returns the device ID.
 */
@property (nonatomic, readonly) NSString *deviceId;

/**
 *  Registers user's device push token, in order to be able to send him notifications from Push Technologies platform.
 *
 *  @param pushToken Data from application delegates's method `application:didRegisterForRemoteNotificationsWithDeviceToken:`
 */
-(void)registerPushToken:(NSData*)pushToken;

/**
 *  Removes any data from the model.
 */
-(void)clean;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @name Campaigns

/**
 *  Returns an array of `PSHCampaignDAO` instances.
 */
@property (nonatomic, readonly) NSArray* campaignList;

/**
 *  Asynchronous callback block to be used on `handleRemotePushWithUserInfo:completion:`.
 *
 *  @param BOOL            `YES` if the notification is a valid push; `NO` otherwise.
 *  @param id               Push info object, or `nil` if any error happens or the notification didn't contain any push related info.
 *  @param NSError*        An error if the operation fails; otherwise (success), `nil`.
 */
typedef void(^PSHHandleRemotePushAsyncBlock)(BOOL, id, NSError*);

/**
 *  Asynchronous callback block to be used on `handleRemotePushWithUserInfo:completionCustom:completionCampaign:completionOther:`.
 *
 *  @param BOOL            `YES` if the notification did contain any NEW campaign related info; `NO` otherwise.
 *  @param PSHCampaignDAO* Campaign info object, or `nil` if any error happens or the notification didn't contain any campaign related info.
 *  @param NSError*        An error if the operation fails; otherwise (success), `nil`.
 */
typedef void(^PSHHandleRemotePushCampaignAsyncBlock)(BOOL, PSHCampaignDAO*);

/**
 *  Asynchronous callback block to be used on `handleRemotePushWithUserInfo:completionCustom:completionCampaign:completionOther:`.
 *
 *  @param BOOL            `YES` if the notification is a valid custom push; `NO` otherwise.
 *  @param PSHCustomDAO*    Custom info object, or `nil` if any error happens or the notification didn't contain any custom related info.
 *  @param NSError*         An error if the operation fails; otherwise (success), `nil`.
 */
typedef void(^PSHHandleRemotePushCustomAsyncBlock)(BOOL, PSHCustomDAO*);

/**
 *  Asynchronous callback block to be used on `handleRemotePushWithUserInfo:completionCustom:completionCampaign:completionOther:`.
 *
 *  @param NSDictionary*    Push dictionary (userInfo)
 */
typedef void(^PSHHandleRemotePushOtherAsyncBlock)(NSDictionary *);

/**
 *  Asynchronous callback block to be used on `handleRemotePushWithUserInfo:completionCustom:completionCampaign:completionOther:`.
 *
 *  @param NSError*         Error.
 */
typedef void(^PSHHandleRemotePushFailAsyncBlock)(NSError *);

/**
 
 This should be called whenever a remote notification is received in order to know if it contains any **new campaign** information and update the model: in `application:didFinishLaunchingWithOptions:` (asking `launchOptions` for `UIApplicationLaunchOptionsRemoteNotificationKey` key) and `application:didReceiveRemoteNotification:` (both `UIApplicationDelegate` methods).
 
 
 `PSHHandleRemotePushAsyncBlock` signature has three parameters (`BOOL`, `PSHCampaignDAO`&#42;, `NSError`&#42;):
 
 +  `YES` if the notification did contain any NEW campaign related info; `NO` otherwise.
 +  Campaign info object, or `nil` if any error happens or the notification didn't contain any campaign related info.
 +  An error if the operation fails; otherwise (success), `nil`.
 
 @param userInfo        Dictionary from the `UIApplicationDelegate` method.
 @param completionBlock Asynchronously executed block, fired once the operation has finished.
 */
-(void)handleRemotePushWithUserInfo:(NSDictionary*)userInfo completion:(PSHHandleRemotePushAsyncBlock)completionBlock;

/**
 
 This should be called whenever a remote notification is received in order to know what kind of push we have received. Depending on it we define up to three different completion blocks: for campaigns, custom notifications and other notifications (non PushTech).
 
 @param userInfo                Dictionary from the `UIApplicationDelegate` method.
 @param completionCampaignBlock Asynchronously executed block, fired once we got a Campaign.
 @param completionCustomBlock   Asynchronously executed block, fired once we got a Custom notification.
 @param completionOtherBlock    Asynchronously executed block, fired once we got other notification.
 @param failBlock               Asynchronously executed block, fired on error getting campaign or custom notification.
 */
-(void)handleRemotePushWithUserInfo:(NSDictionary*)userInfo
                   completionCustom:(PSHHandleRemotePushCustomAsyncBlock)completionCustomBlock
                 completionCampaign:(PSHHandleRemotePushCampaignAsyncBlock)completionCampaignBlock
                    completionOther:(PSHHandleRemotePushOtherAsyncBlock)completionOtherBlock
                               fail:(PSHHandleRemotePushFailAsyncBlock)failBlock;

/**
 *  Mark the campaign as viewed (updating the model and informing Push Technologies SDK platform).
 *
 *  @param campaign The viewed campaign.
 */
-(void)markCampaignAsViewed:(PSHCampaignDAO *)campaign;

@end